package com.honter.ehcache;


import com.google.common.hash.Hashing;
import com.honter.config.LogConfig;
import org.apache.ibatis.jdbc.Null;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.ehcache.EhCacheCacheManager;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.util.ClassUtils;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.nio.charset.Charset;

/**
 * Created by Smart on 2018/3/16.
 */
@Configuration
@EnableCaching
//@Profile("cache")
public class CacheConfig extends CachingConfigurerSupport {
    private static final Logger LOGGER = LoggerFactory.getLogger(LogConfig.class);
    /**
     * 缓存配置文件路径
     */
    @Value("${spring.cache.ehcache.config}")
    String myCacheLocation;

    /**
     * ehcache缓存
     */
    @Bean
    @Override
    public CacheManager cacheManager() {

        try {
            return new EhCacheCacheManager(
                    net.sf.ehcache.CacheManager.newInstance(
                            new PathMatchingResourcePatternResolver().getResources(this.myCacheLocation)[0].getURL()
                    ));
        } catch (Exception ex) {
            LOGGER.error("ehcache缓存获取异常："+ex.getMessage());
            return null;
        }
    }

    /**
     * 自动生成缓存健值
     */
    @Bean
    @Override
    public KeyGenerator keyGenerator() {
        return new KeyGenerator() {
            @Override
            public Object generate(Object target, Method method, Object... params) {
                StringBuilder key = new StringBuilder();
                key.append(target.getClass().getSimpleName()).append(".").append(method.getName()).append(":");
                if (params.length == 0) {
                    //NO_PARAM_KEY
                    return key.append(0).toString();
                }
                for (Object param : params) {
                    if (param == null) {
//                log.warn("input null param for Spring cache, use default key={}", NULL_PARAM_KEY);
                        //NULL_PARAM_KEY
                        key.append(53);
                    } else if (ClassUtils.isPrimitiveArray(param.getClass())) {
                        int length = Array.getLength(param);
                        for (int i = 0; i < length; i++) {
                            key.append(Array.get(param, i));
                            key.append(',');
                        }
                    } else if (ClassUtils.isPrimitiveOrWrapper(param.getClass()) || param instanceof String) {
                        key.append(param);
                    } else {
//                log.warn("Using an object as a cache key may lead to unexpected results. " +
//                        "Either use @Cacheable(key=..) or implement CacheKey. Method is " + target.getClass() + "#" + method.getClassName());
                        key.append(param.hashCode());
                    }
                    key.append('-');
                }

                String finalKey = key.toString();
                long cacheKeyHash = Hashing.murmur3_128().hashString(finalKey, Charset.defaultCharset()).asLong();
//        log.debug("using cache key={} hashCode={}", finalKey, cacheKeyHash);
                return key.toString();
            }
        };
    }

}