package com.honter.security;

import com.alibaba.fastjson.JSONObject;
import com.honter.config.ConfigBeanValue;
import io.jsonwebtoken.Jwts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Shuangyao
 * 22:55 2018/10/15
 */
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private Logger logger = LoggerFactory.getLogger(JwtAuthenticationFilter.class);

    @Autowired
    private JwtTokenProvider jwtTokenProvider;

    @Autowired
    private ConfigBeanValue configBeanValue;

    @Autowired
    private UserAuthService userAuthService;

    //1.从每个请求header获取token
    //2.调用前面写的validateToken方法对token进行合法性验证
    //3.解析得到username，并从database取出用户相关信息权限
    //4.把用户信息以UserDetail形式放进SecurityContext以备整个请求过程使用。
    // （例如哪里需要判断用户权限是否足够时可以直接从SecurityContext取出去check
    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        String token = getJwtFromRequest(request);

        if (token != null && jwtTokenProvider.validateToken(token)) {
            Map<String, Object> map = new HashMap<String, Object>();
            map = getUsernameFromJwt(token);
            String username = map!=null?(map.get("username")!=null?(String)map.get("username"):""):"";
            UserDetails userDetails = userAuthService.getUserDetailByUserName(username);
            //SecurityContextHolder.getContext().getAuthentication().getCredentials()获取map得值
            Authentication authentication = new UsernamePasswordAuthenticationToken(
                    userDetails, map, userDetails.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authentication);
            //自动刷新token并保存到Reponse中
            AutomaticallyRefreshToken(map,response);
        } else {
            logger.error(request.getParameter("username") + " :Token is null");
        }
        super.doFilter(request, response, filterChain);
    }

    private String getJwtFromRequest(HttpServletRequest request) {
        String token = request.getHeader("Authorization");
        if (token != null && token.startsWith("Bearer")) {
            return token.replace("Bearer ", "");
        }
        return null;
    }

    private Map<String, Object> getUsernameFromJwt(String token) {
        String mapstr=Jwts.parser().setSigningKey(configBeanValue.getJwtTokenSecret())
                .parseClaimsJws(token)
                .getBody()
                .getSubject();
        Map<String, Object> map = new HashMap<String, Object>();
        map=(Map) JSONObject.parse(mapstr);
        return map;
    }

    private void AutomaticallyRefreshToken(Map<String, Object> map,HttpServletResponse response){
        if(map!=null&&map.get("date")!=null){
            Long dd=(Long)map.get("date");
            Date now=new Date();
            if(now.getTime()-dd>configBeanValue.getTokenRefreshMs()){
                map.put("date",now.getTime());
                response.setHeader("Authorization","Bearer "+jwtTokenProvider.createJwtTokenContMap(map));
            }
        }else{
            map.put("date",new Date().getTime());
            response.setHeader("Authorization","Bearer "+jwtTokenProvider.createJwtTokenContMap(map));
        }
    }
}
