package com.honter.serviceImpl;

import com.honter.mapper.UserMapper;
import com.honter.model.User;
import com.honter.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by admin on 2019/3/5.
 */
@EnableTransactionManagement
@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private UserMapper userMapper;
    public User queryUserByUserId(Integer userId){
        SimpleDateFormat ft = new SimpleDateFormat ("yyyy.MM.dd hh:mm:ss");
        Date now = new Date( );
        System.out.println("开始时间："+ft.format(now));
        try {
            Thread.sleep(3);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        Date nowOver = new Date( );
        System.out.println("结束时间："+ft.format(nowOver));
        User user=userMapper.selectByPrimaryKey(userId);
        return user;
    }

    /**
     *  Isolation.READ_UNCOMMITTED读取未提交数据(会出现脏读, 不可重复读) 基本不使用
     *  Isolation.READ_COMMITTED读取已提交数据(会出现不可重复读和幻读)
     *  Isolation.REPEATABLE_READ可重复读(会出现幻读)
     *  Isolation.SERIALIZABLE串行化

     MYSQL: 默认为REPEATABLE_READ级别
     SQLSERVER: 默认为READ_COMMITTED

     脏读 : 一个事务读取到另一事务未提交的更新数据
     不可重复读 : 在同一事务中, 多次读取同一数据返回的结果有所不同, 换句话说, 
     后续读取可以读到另一事务已提交的更新数据. 相反, "可重复读"在同一事务中多次
     读取数据时, 能够保证所读数据一样, 也就是后续读取不能读到另一事务已提交的更新数据
     幻读 : 一个事务读到另一个事务已提交的insert数据

     @Transactional 只能被应用到public方法上, 对于其它非public的方法,
     如果标记了@Transactional也不会报错,但方法没有事务功能
     @Transactional可以放在接口的方法中
     */
    public Integer insertUser(User user){
        userMapper.insertSelective(user);
//        String str=null;
//        str.equals("ss");
        return user.getUserId();
    }
    @Transactional(isolation = Isolation.REPEATABLE_READ)
    public void updateUserByUserId(User user){
        User user33=userMapper.selectByPrimaryKey(user.getUserId());
        System.out.println(user33.getPhone());
        SimpleDateFormat ft = new SimpleDateFormat ("yyyy.MM.dd hh:mm:ss");
        Date now = new Date( );
        System.out.println("开始时间："+ft.format(now));
        try {
            Thread.sleep(3);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        Date nowOver = new Date( );
        System.out.println("结束时间："+ft.format(nowOver));
        User user44=userMapper.selectByPrimaryKey(user.getUserId());
        System.out.println(user44.getPhone());
        userMapper.updateByPrimaryKeySelective(user);
    }
    public List<User> getUsersByuserId(Integer userId,String userName){
        List<User> list=new ArrayList<User>();
        list=userMapper.getUsersByuserId(userId,userName);
        return  list;
    }

    @Override
    public User getUsersByUserName(String userName) {
        return userMapper.getUsersByUserName(userName);
    }
}
